package com.dbxml.db.admin.nodes;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CollectionsNode.java,v 1.6 2004/02/12 00:17:54 bradford Exp $
 */

import com.dbxml.db.admin.Admin;
import com.dbxml.db.client.CollectionClient;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import com.dbxml.db.admin.dialogs.CollectionDialog;

/**
 * CollectionsNode
 */

public final class CollectionsNode extends AdminNode implements HasCollection, HasChildren, HasMenu {
   private static final AdminNode[] EmptyAdminNodes = new AdminNode[0];
   private static final Icon CollectionsIcon = new ImageIcon(CollectionNode.class.getResource("databases_mini.gif"));

   private static final int ACTION_ADD = 0;

   private static final MenuItem[] Menu = new MenuItem[]{
      new MenuItem(ACTION_ADD, "Add Collection"),
   };

   private CollectionClient col;

   public CollectionsNode(AdminNode parentNode, CollectionClient col) {
      super(parentNode);

      this.col = col;
   }

   public String getLabel() {
      return "Collections";
   }

   public Icon getIcon() {
      return CollectionsIcon;
   }

   public CollectionClient getCollection() {
      return col;
   }

   public AdminNode[] getChildren() {
      try {
         List list = new ArrayList();
         try {
            String[] collections = col.listCollections();
            for ( int j = 0; j < collections.length; j++ )
               list.add(new CollectionNode(this, col, collections[j]));
         }
         catch ( Exception e ) {
            Admin.getInstance().addMessage(e.toString());
         }

         return (AdminNode[])list.toArray(EmptyAdminNodes);
      }
      catch ( Exception e ) {
         return EmptyAdminNodes;
      }
   }

   public MenuItem[] getMenu() {
      return Menu;
   }

   public int menuAction(int action) {
      Admin admin = Admin.getInstance();

      switch ( action ) {
         case ACTION_ADD: {
            CollectionDialog cd = new CollectionDialog(admin, col);
            cd.show();
            if ( cd.isOkClicked() )
               return REFRESH_SELF;
         }
      }
      return REFRESH_NONE;
   }
}
