package com.dbxml.db.admin.nodes;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: IndexesNode.java,v 1.4 2004/02/12 00:17:54 bradford Exp $
 */

import com.dbxml.db.admin.Admin;
import com.dbxml.db.admin.dialogs.IndexDialog;
import com.dbxml.db.client.CollectionClient;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * IndexesNode
 */

public final class IndexesNode extends AdminNode implements HasCollection, HasChildren, HasMenu {
   private static final AdminNode[] EmptyAdminNodes = new AdminNode[0];
   private static final Icon IndexesIcon = new ImageIcon(CollectionNode.class.getResource("indexes_mini.gif"));

   public static final int ACTION_ADD = 0;

   private static final MenuItem[] Menu = new MenuItem[]{
      new MenuItem(ACTION_ADD, "Add Index"),
   };

   private CollectionClient col;

   public IndexesNode(AdminNode parentNode, CollectionClient col) {
      super(parentNode);

      this.col = col;
   }

   public String getLabel() {
      return "Indexes";
   }

   public Icon getIcon() {
      return IndexesIcon;
   }

   public CollectionClient getCollection() {
      return col;
   }

   public AdminNode[] getChildren() {
      try {
         String[] idx = col.listIndexers();
         AdminNode[] list = new AdminNode[idx.length];
         for ( int i = 0; i < idx.length; i++ )
            list[i] = new IndexNode(this, col, idx[i]);

         return list;
      }
      catch ( Exception e ) {
         return EmptyAdminNodes;
      }
   }

   public MenuItem[] getMenu() {
      return Menu;
   }

   public int menuAction(int action) {
      Admin admin = Admin.getInstance();

      switch ( action ) {
         case ACTION_ADD: {
            IndexDialog id = new IndexDialog(admin, col);
            id.show();
            if ( id.isOkClicked() )
               return REFRESH_SELF;
         }
      }
      return REFRESH_NONE;
   }
}
