package com.dbxml.db.admin.nodes;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: UserNode.java,v 1.3 2004/02/12 00:17:54 bradford Exp $
 */

import com.dbxml.db.admin.Admin;
import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.common.security.AccessManagerClient;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

/**
 * UserNode
 */

public final class UserNode extends AdminNode implements HasMenu, HasChildren, HasCollection {
   private static final Icon UserIcon = new ImageIcon(UserNode.class.getResource("user_mini.gif"));

   public static final int ACTION_REMOVE = 0;

   private CollectionClient col;
   private String id;

   public UserNode(AdminNode parentNode, CollectionClient col, String id) {
      super(parentNode);

      this.col = col;
      this.id = id;
   }

   public String getLabel() {
      return id;
   }

   public Icon getIcon() {
      return UserIcon;
   }

   public CollectionClient getCollection() {
      return col;
   }

   public AdminNode[] getChildren() {
      AdminNode[] list = new AdminNode[1];
      list[0] = new RolesNode(this, col, id);
      return list;
   }

   public MenuItem[] getMenu() {
      return new MenuItem[] {new MenuItem(ACTION_REMOVE, "Remove User")};
   }

   public String getMenuTitle() {
      return "User";
   }

   public int menuAction(int action) {
      switch ( action ) {
         case ACTION_REMOVE:
            try {
               String title = "Remove User";
               String message = "Are you sure you want to remove the\n"
                              + "User '"+id+"'?";
               int res = JOptionPane.showConfirmDialog(Admin.getInstance(), message, title, JOptionPane.YES_NO_OPTION);
               if ( res == JOptionPane.YES_OPTION ) {
                  AccessManagerClient manager = new AccessManagerClient(col.getClient());
                  manager.removeUser(id);
                  return REMOVE_SELF;
               }
            }
            catch ( Exception e ) {
               Admin.getInstance().addMessage(e.getMessage());
            }
            break;

      }
      return REFRESH_NONE;
   }
}
