package com.dbxml.db.client.local;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ResultSetClientImpl.java,v 1.2 2004/02/12 00:17:55 bradford Exp $
 */

import org.w3c.dom.Node;

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.ResultSetClient;
import com.dbxml.db.core.query.ResultSet;
import com.dbxml.util.dbXMLException;
import com.dbxml.xml.dtsm.DTSMHelper;
import com.dbxml.xml.dtsm.DocumentTable;

/**
 * ResultSetClientImpl
 */

public final class ResultSetClientImpl implements ResultSetClient {
   private CollectionClient col;
   private ResultSet rs;

   public ResultSetClientImpl(CollectionClient col, ResultSet rs) {
      this.col = col;
      this.rs = rs;
   }

   public ResultSet getInternalResultSet() {
      return rs;
   }

   public CollectionClient getCollection() throws dbXMLException {
      return col;
   }

   public String getQueryStyle() throws dbXMLException {
      return rs.getQuery().getQueryStyle();
   }

   public String getQueryString() throws dbXMLException {
      return rs.getQuery().getQueryString();
   }

   public boolean next() throws dbXMLException {
      return rs.next();
   }

   public void close() throws dbXMLException {
      rs.close();
   }

   public int getCount() throws dbXMLException {
      return rs.getCount();
   }

   public Node getResult() throws dbXMLException {
      DocumentTable dt = rs.getResult();
      return DTSMHelper.tableToNode(dt);
   }

   public String getResultAsText() throws dbXMLException {
      DocumentTable dt = rs.getResult();
      return DTSMHelper.tableToText(dt);
   }

   public CollectionClient getResultCollection() throws dbXMLException {
      return new CollectionClientImpl(col.getClient(), rs.getResultCollection());
   }

   public String getResultKey() throws dbXMLException {
      return rs.getResultKey().toString();
   }
}
