package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ContentDrop.java,v 1.5 2004/02/12 00:17:55 bradford Exp $
 */

import java.io.PrintWriter;

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.tools.CommandLine;
import com.dbxml.util.dbXMLException;

/**
 * ContentDrop (RM) removes content from the current context Collection.
 */

public final class ContentDrop extends ContentBase {
   protected void process(CollectionClient col, String docName) throws dbXMLException {
      boolean wildcard = docName.indexOf('?') != -1 || docName.indexOf('*') != -1;
      PrintWriter pw = cl.getWriter();

      if ( cl.isPropertyTrue(CommandLine.CONFIRM) ) {
         try {
            String ans;
            if ( wildcard )
               ans = cl.readLine("Delete Document(s) (y/N)? ");
            else
               ans = cl.readLine("Delete Document (y/N)? ");

            if ( !ans.equalsIgnoreCase("yes") && !ans.equalsIgnoreCase("y") ) {
               pw.println("Document deletion canceled");
               return;
            }
         }
         catch ( Exception e ) {
            throw new dbXMLException("Error obtaining answer");
         }
      }

      if ( wildcard ) {
         WildcardMatcher matcher = new WildcardMatcher(docName);
         String[] list = col.listKeys();
         int count = 0;
         int error = 0;
         for ( int i = 0; i < list.length; i++ ) {
            String key = list[i];
            if ( matcher.matches(key) ) {
               try {
                  col.remove(key);
                  count++;
               }
               catch ( Exception e ) {
                  pw.println("Document '" + key + "' not deleted");
                  error++;
               }
            }
         }

         if ( cl.isPropertyTrue(CommandLine.TOTALS) ) {
            if ( error > 0 ) {
               pw.println("-----------------------");
               pw.println(count+" Document(s) deleted ("+error+" errors)");
            }
            else
               pw.println(count+" Document(s) deleted");
         }
      }
      else {
         try {
            col.remove(docName);
            pw.println("Document '" + docName + "' deleted");
         }
         catch ( Exception e ) {
            pw.println("Document '" + docName + "' not deleted: "+e.getMessage());
         }
      }
   }
}

