package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CreateBase.java,v 1.6 2004/02/12 00:17:55 bradford Exp $
 */

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.tools.CommandLine;
import com.dbxml.util.dbXMLException;
import java.io.PrintWriter;
import java.util.Properties;

/**
 * CreateBase is a base class for several creation-related Commands.
 */

public abstract class CreateBase extends CommandBase {
   public final void process() throws dbXMLException {
      CollectionClient col = (CollectionClient)cl.getProperty(CommandLine.COLLECTION);
      if ( col == null )
         throw new dbXMLException("Collection context required");

      String name = cl.getNextToken(getType()+" Name");
      Properties props = cl.parseProperties();

      PrintWriter pw = cl.getWriter();

      if ( cl.isPropertyTrue(CommandLine.CONFIRM) ) {
         try {
            String ans = cl.readLine("Create " + getType() + " (Y/n)? ");
            if ( ans.equalsIgnoreCase("no") || ans.equalsIgnoreCase("n") ) {
               pw.println(getType() + " creation canceled");
               return;
            }
         }
         catch ( Exception e ) {
            throw new dbXMLException("Error obtaining answer");
         }
      }

      if ( createObject(col, name, props) )
         pw.println(getType() + " '" + name + "' created");
      else
         pw.println(getType() + " '" + name + "' not created");
   }


   protected abstract String getType();

   protected abstract boolean createObject(CollectionClient col, String name, Properties props) throws dbXMLException;
}

