package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CreateIndex.java,v 1.9 2004/02/12 00:17:55 bradford Exp $
 */

import java.util.Properties;

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.server.dbXML;
import com.dbxml.util.dbXMLException;
import com.dbxml.xml.dom.DOMHelper;
import java.io.PrintWriter;
import com.dbxml.db.client.tools.CommandLine;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.util.Iterator;
import java.util.Map;

/**
 * CreateIndex (MKIDX) creates a new Index under the current context
 * Collection.
 */

public class CreateIndex extends CreateBase {
   private static final String CLASS = "class";
   private static final String DEFAULT_CLASS = dbXML.DEFAULT_VALUE_INDEXER;
   private static final String PAGESIZE = "pagesize";
   private static final String DEFAULT_PAGESIZE = "4096";
   private static final String PATTERN = "pattern";
   private static final String TYPE = "type";
   private static final String DEFAULT_TYPE = "string";

   private static final String NAME_TYPE = "name";
   private static final String NAME_CLASS = dbXML.DEFAULT_NAME_INDEXER;
   private static final String TEXT_TYPE = "fulltext";
   private static final String TEXT_CLASS = dbXML.DEFAULT_TEXT_INDEXER;


   protected String getType() {
      return "Index";
   }

   protected boolean createObject(CollectionClient col, String name, Properties props) throws dbXMLException {
      try {
         String cname = props.getProperty(CLASS, DEFAULT_CLASS);
         String psize = props.getProperty(PAGESIZE, DEFAULT_PAGESIZE);
         String pattern = props.getProperty(PATTERN);
         if ( pattern == null || pattern.length() == 0 )
            pattern = name;

         String propClass = props.getProperty(CLASS);
         String type = props.getProperty(TYPE, DEFAULT_TYPE);

         props.remove(PAGESIZE);
         props.remove(PATTERN);
         props.remove(CLASS);
         props.remove(TYPE);

         if ( propClass == null || propClass.length() == 0 ) {
            if ( type.equalsIgnoreCase(NAME_TYPE) ) {
               type = DEFAULT_TYPE;
               cname = NAME_CLASS;
            }
            else if ( type.equalsIgnoreCase(TEXT_TYPE) ) {
               type = DEFAULT_TYPE;
               cname = TEXT_CLASS;
            }
         }

         Document doc = DOMHelper.newDocument();
         Element root = doc.createElement("index");
         root.setAttribute("name", name);
         root.setAttribute("class", cname);
         root.setAttribute("pattern", pattern);
         root.setAttribute("type", type);
         root.setAttribute("pagesize", psize);
         doc.appendChild(root);

         Iterator iter = props.entrySet().iterator();
         while ( iter.hasNext() ) {
            Map.Entry entry = (Map.Entry)iter.next();
            String key = (String)entry.getKey();
            String value = (String)entry.getValue();
            root.setAttribute(key, value);
         }

         boolean result = col.createIndexer(doc) != null;

         if ( result && cl.isPropertyTrue(CommandLine.TOTALS) ) {
            PrintWriter pw = cl.getWriter();

            int colType = col.getCollectionType();
            String typeName;
            if ( colType == CollectionClient.TYPE_DOCUMENTS )
               typeName = "Document";
            else
               typeName = "Value";

            long count = col.getKeyCount();
            if ( count != 1 )
               pw.println(count+" "+typeName+"s queued for indexing");
            else
               pw.println(count+" "+typeName+" queued for indexing");
         }

         return result;
      }
      catch ( Exception e ) {
         throw new dbXMLException(e);
      }
   }
}




