package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ListBase.java,v 1.6 2004/02/12 00:17:55 bradford Exp $
 */

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.tools.CommandLine;
import com.dbxml.util.dbXMLException;
import java.io.PrintWriter;

/**
 * ListBase serves as a base class for several listing Commands.
 */

public abstract class ListBase extends CommandBase {
   public final void process() throws dbXMLException {
      CollectionClient col = (CollectionClient)cl.getProperty(CommandLine.COLLECTION);
      if ( col == null )
         throw new dbXMLException("Collection context required");

      WildcardMatcher matcher = null;
      if ( cl.hasMoreTokens() )
         matcher = new WildcardMatcher(cl.getNextToken("Filespec"));

      String[] list = getList(col);
      PrintWriter pw = cl.getWriter();
      int count = 0;
      for ( int i = 0; i < list.length; i++ ) {
         if ( matcher == null || matcher.matches(list[i]) ) {
            pw.println(list[i]);
            count++;
         }
      }

      if ( cl.isPropertyTrue(CommandLine.TOTALS) ) {
         if ( count > 0 )
            pw.println("-----------------------");
         if ( count != 1 )
            pw.println(count + " " + getType() + "s");
         else
            pw.println(count + " " + getType());
      }
   }

   protected abstract String getType();

   protected abstract String[] getList(CollectionClient col) throws dbXMLException;
}
