package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: User.java,v 1.5 2004/02/12 00:17:55 bradford Exp $
 */

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.tools.CommandLine;
import com.dbxml.db.common.security.AccessManagerClient;
import com.dbxml.util.dbXMLException;
import java.io.PrintWriter;

/**
 * User (USEr) provides several User management functions.
 */

public final class User extends CommandBase {
   private static final String ACTION_PASSWORD = "PASSWORD";
   private static final String ACTION_ADDROLE = "ADDROLE";
   private static final String ACTION_RMROLE = "RMROLE";
   private static final String ACTION_LSROLE = "LSROLE";

   public void process() throws dbXMLException {
      CollectionClient col = (CollectionClient)cl.getProperty(CommandLine.COLLECTION);
      if ( col == null )
         throw new dbXMLException("Collection context required");

      AccessManagerClient manager = new AccessManagerClient(cl.getClient());
      PrintWriter pw = cl.getWriter();

      if ( !cl.hasMoreTokens() )
         throw new dbXMLException("User ID is required");
      String userID = cl.getNextToken();

      if ( !cl.hasMoreTokens() )
         throw new dbXMLException("A User action must be specified");
      String action = cl.getNextToken().toUpperCase();

      if ( action.equals(ACTION_PASSWORD) ) {
         if ( !cl.hasMoreTokens() )
            throw new dbXMLException("A new password must be specified");
         String password = cl.getNextToken();
         manager.setUserPassword(userID, password);
         pw.println("Password set for '"+userID+"'");
      }
      else if ( action.equals(ACTION_ADDROLE) ) {
         if ( !cl.hasMoreTokens() )
            throw new dbXMLException("A Role ID must be specified");
         String roleID = cl.getNextToken();
         manager.addRoleToUser(userID, roleID);
         pw.println("Role '"+roleID+"' added to User '"+userID+"'");
      }
      else if ( action.equals(ACTION_RMROLE) ) {
         if ( !cl.hasMoreTokens() )
            throw new dbXMLException("A Role ID must be specified");
         String roleID = cl.getNextToken();
         manager.removeRoleFromUser(userID, roleID);
         pw.println("Role '"+roleID+"' removed from User '"+userID+"'");
      }
      else if ( action.equals(ACTION_LSROLE) ) {
         String[] list = manager.listRolesForUser(userID);
         int count = 0;
         for ( int i = 0; i < list.length; i++ ) {
            pw.println(list[i]);
            count++;
         }

         if ( cl.isPropertyTrue(CommandLine.TOTALS) ) {
            if ( count > 0 )
               pw.println("-----------------------");
            if ( count != 1 )
               pw.println(count + " Roles");
            else
               pw.println(count + " Role");
         }
      }
      else
         throw new dbXMLException("Unknown action '"+action+"'");
   }
}

