package com.dbxml.db.client.xmldb;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: BinaryResourceImpl.java,v 1.2 2004/02/12 00:17:55 bradford Exp $
 */

import com.dbxml.util.UTF8;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ErrorCodes;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.BinaryResource;

/**
 * BinaryResourceImpl
 */

public final class BinaryResourceImpl implements BinaryResource {
   public static final String TYPE = "BinaryResource";

   private CollectionImpl parent;
   private String id;
   private byte[] content;

   public BinaryResourceImpl(CollectionImpl parent, String id, byte[] content) {
      this.parent = parent;
      this.id = id;
      this.content = content;
   }

   public Collection getParentCollection() throws XMLDBException {
      return parent;
   }

   public String getId() throws XMLDBException {
      return id;
   }

   public String getResourceType() throws XMLDBException {
      return TYPE;
   }

   public Object getContent() throws XMLDBException {
      return content;
   }

   public void setContent(Object value) throws XMLDBException {
      if ( value instanceof byte[] )
         content = (byte[])value;
      else if ( value instanceof String )
			content = UTF8.toUTF8((String)value);
      else if ( value != null )
         throw new XMLDBException(ErrorCodes.WRONG_CONTENT_TYPE, "Setting content as '" + value.getClass().getName() + "' is not supported");
      else
         throw new XMLDBException(ErrorCodes.WRONG_CONTENT_TYPE, "Setting content as null is not supported");
   }
}
