package com.dbxml.db.common.btree;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CompoundNEQ.java,v 1.2 2004/02/12 00:17:56 bradford Exp $
 */

import com.dbxml.db.core.data.Value;
import com.dbxml.db.core.indexer.IndexPattern;
import com.dbxml.db.core.indexer.IndexQuery;

/**
 * CompoundNEQ is a specialized IndexQuery implementation for use by the
 * BTree class' query method.
 */

final class CompoundNEQ extends IndexQuery {
   public CompoundNEQ(IndexPattern pattern, Value v) {
      this.pattern = pattern;
      this.op = NEQ;

      // Must morph it into a null terminated starts-with
      byte[] r = v.getRawData();
      int pos = v.getOffset();
      int len = v.getLength();

      byte[] b = new byte[len+2];
      System.arraycopy(r, pos, b, 0, len);
      b[b.length-1] = (byte)0xFF; // Was 127
      b[b.length-2] = 0;

      Value v1 = new Value(b, 0, b.length-1);
      Value v2 = new Value(b, 0, b.length-2);
      Value v3 = new Value(b, 0, b.length);

      vals = new Value[]{v1, v2, v3};
   }

   public int getOperator() {
      return NEQ;
   }

   public boolean testValue(Value v) {
      return !v.equals(vals[1]);
   }
}
