package com.dbxml.db.common.query;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ResultSetBase.java,v 1.2 2004/02/12 00:17:56 bradford Exp $
 */

import com.dbxml.db.core.Collection;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.query.ProcessingException;
import com.dbxml.db.core.query.Query;
import com.dbxml.db.core.query.QueryException;
import com.dbxml.db.core.query.ResultSet;
import com.dbxml.db.core.transaction.Transaction;
import com.dbxml.xml.SymbolTable;

/**
 * ResultSetBase
 */

public abstract class ResultSetBase implements ResultSet {
   protected Collection context;
   protected Query query;

   protected Transaction tx;
   protected Key[] keySet;
   protected Key currentKey;
   protected int keyPos;

   protected SymbolTable symbols;
   protected boolean opened;

   public ResultSetBase(Transaction tx, Collection context, Query query, Key[] keySet) {
      this.tx = tx;
      this.context = context;
      this.query = query;
      this.keySet = keySet;

      opened = keySet != null;

      try {
         symbols = context.getSymbols();
      }
      catch ( Exception e ) {
         // Should never happen
      }
   }

   protected void checkOpened() throws QueryException {
      if ( keySet == null )
         throw new ProcessingException("No results");
      if ( !opened )
         throw new ProcessingException("ResultSet is not opened");
   }

   public Collection getCollection() {
      return context;
   }

   public Query getQuery() {
      return query;
   }

   public int getCount() throws QueryException {
      checkOpened();
      return keySet.length;
   }

   public Collection getResultCollection() throws QueryException {
      checkOpened();
      return context;
   }

   public Key getResultKey() throws QueryException {
      checkOpened();
      return currentKey;
   }

   protected Key nextKey() {
      if ( keyPos < keySet.length )
         currentKey = keySet[keyPos++];
      else
         currentKey = null;
      return currentKey;
   }

   public void close() {
      opened = false;
   }
}
