package com.dbxml.db.common.xpath;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: XPathResultSet.java,v 1.6 2004/02/12 00:17:56 bradford Exp $
 */

import com.dbxml.db.common.adapters.DOMAdapter;
import com.dbxml.db.common.query.ResultSetBase;
import com.dbxml.db.core.Collection;
import com.dbxml.db.core.DBException;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.query.ProcessingException;
import com.dbxml.db.core.query.Query;
import com.dbxml.db.core.query.QueryException;
import com.dbxml.db.core.transaction.Transaction;
import com.dbxml.xml.SymbolTable;
import com.dbxml.xml.dtsm.DTSMException;
import com.dbxml.xml.dtsm.DTSMHelper;
import com.dbxml.xml.dtsm.DocumentTable;
import org.apache.xml.utils.PrefixResolver;
import org.apache.xml.utils.PrefixResolverDefault;
import org.apache.xpath.XPath;
import org.apache.xpath.XPathContext;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import com.dbxml.xml.dom.DOMUtils;
import org.w3c.dom.Element;

/**
 * XPathResultSet
 */

final class XPathResultSet extends ResultSetBase {
   private SymbolTable symbols;
   private String xpath;
   private PrefixResolver pr;
   private XPath xp;

   private NodeIterator ni;
   private DocumentTable dt;
   private DOMAdapter domAdapter;
   private Node node;

   public XPathResultSet(Transaction tx, Collection context, Query query, Key[] keySet, PrefixResolver pr, String xpath) {
      super(tx, context, query, keySet);
      this.pr = pr;
      this.xpath = xpath;
      domAdapter = new DOMAdapter(context);
   }

   private void prepareNextNode() throws QueryException {
      node = null;
      while ( nextKey() != null ) {
         Transaction tx = new Transaction();
         try {
            Document d = domAdapter.getDocument(tx, currentKey);

            if ( d == null )
               continue;

            Node n = d.getDocumentElement();

            XPathContext xpc = new XPathContext();
            PrefixResolver pfx;
            if ( pr == null ) {
               pfx = new PrefixResolverDefault(d.getDocumentElement());
               xp = new XPath(xpath, null, pfx, XPath.SELECT, null);
            }
            else {
               pfx = pr;
               if ( xp == null )
                  xp = new XPath(xpath, null, pfx, XPath.SELECT, null);
            }

            ni = xp.execute(xpc, n, pfx).nodeset();

            node = ni.nextNode();
            if ( node != null )
               break;
         }
         catch ( Exception e ) {
            try {
               tx.cancel();
            }
            catch ( DBException ex ) {
               ex.printStackTrace(System.err);
            }
         }
         finally {
            if ( tx.getStatus() == Transaction.ACTIVE ) {
               try {
                  tx.commit();
               }
               catch ( DBException e ) {
                  e.printStackTrace(System.err);
               }
            }
         }
      }
   }

   public int getResultType() throws QueryException {
      checkOpened();
      getResult();
      if ( dt != null )
         return dt.getEntry(0).getTypeID();
      else
         return -1;
   }

   public int getCount() throws QueryException {
      checkOpened();
      return -1;
   }

   public boolean next() throws QueryException {
      checkOpened();
      dt = null;
      if ( ni == null )
         prepareNextNode();
      else {
         node = ni.nextNode();
         if ( node == null )
            prepareNextNode();
      }
      return node != null;
   }

   public DocumentTable getResult() throws QueryException {
      checkOpened();
      if ( dt == null && node != null ) {
         try {
				if ( node.getNodeType() == Node.ELEMENT_NODE )
					DOMUtils.importNamespaces((Element)node);
				
            dt = DTSMHelper.nodeToTable(node, symbols);
         }
         catch ( DTSMException e ) {
            throw new ProcessingException(e);
         }
      }
      return dt;
   }
}

