package com.dbxml.db.core;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ClassResolver.java,v 1.2 2004/02/12 00:17:56 bradford Exp $
 */

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * ClassResolver is a utility class to replaces the use of
 * Class.forName for dynamic class loading.  It allows
 * classes that are registered with it to be aliased. It also
 * allows some classes to be early bound, so that dbXML can
 * be compiled into a native executable without late binding
 * issues.
 * <br />
 * Its purpose is to allow for dbXML to grow, and possibly even
 * repackage without affecting existing database configuration
 * settings.
 */

public final class ClassResolver {
   private static final Map classes = Collections.synchronizedMap(new HashMap());
   
   private ClassResolver() {
   }
   
   public static Class get(String name) throws ClassNotFoundException {
      Class c = (Class)classes.get(name);
      if ( c != null )
         return c;
      else
         return Class.forName(name);
   }
   
   public static void register(Class c) {
      register(c.getName(), c);
   }
   
   public static void register(String name, Class c) {
      classes.put(name, c);
   }
}


