package com.dbxml.db.enterprise.jsp;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CollectionTag.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.dbXMLClient;
import com.dbxml.util.dbXMLException;

/**
 * CollectionTag creates a reference to a CollectionClient.
 * <br>
 * <br>
 * Example:
 * <br>
 * <pre>   &lt;dbxml:collection name="myCol" connection="myCon" path="/somecollection"/></pre>
 */

public class CollectionTag extends TagSupport {
   static final String NAME = "name";
   static final String CONNECTION = "connection";
   static final String PATH = "path";

   private String name;
   private String connection;
   private String path;

   public CollectionTag() {
   }

   public String getName() {
      return name;
   }

   public void setName(String name) {
      this.name = name;
   }

   public String getConnection() {
      return connection;
   }

   public void setConnection(String connection) {
      this.connection = connection;
   }

   public String getPath() {
      return path;
   }

   public void setPath(String path) {
      this.path = path;
   }

   public int doStartTag() throws JspException {
      Object obj = pageContext.getAttribute(connection);
      if ( obj == null && !(obj instanceof dbXMLClient) )
         throw new JspException("No Connection named '" + connection + "' has been declared");
      dbXMLClient client = (dbXMLClient)obj;

      try {
         CollectionClient col = client.getCollection(path);
         pageContext.setAttribute(name, col);
      }
      catch ( dbXMLException e ) {
         throw new JspException("Can't connect to Collection '" + path + "' on Connection '" + connection + "'", e);
      }

      return SKIP_BODY;
   }
}
