package com.dbxml.db.enterprise.jsp;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: GetDocumentTag.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import com.dbxml.db.client.CollectionClient;

/**
 * GetDocumentTag creates a reference to a Document or streams its contents
 * to the JspWriter.
 * <br>
 * <br>
 * Example:
 * <br>
 * <pre>   &lt;dbxml:get name="myDoc" collection="myCol" key="myDoc.xml"/></pre>
 */

public class GetDocumentTag extends TagSupport {
   static final String NAME = "name";
   static final String COLLECTION = "collection";
   static final String KEY = "key";
   static final String PREFIX = "prefix";
   static final String TEXT = "text";

   private String name;
   private String collection;
   private String key;
   private boolean text;
   private boolean prefix = true;

   public GetDocumentTag() {
   }

   public String getName() {
      return name;
   }

   public void setName(String name) {
      this.name = name;
   }

   public String getCollection() {
      return collection;
   }

   public void setCollection(String collection) {
      this.collection = collection;
   }

   public String getKey() {
      return key;
   }

   public void setKey(String key) {
      this.key = key;
   }

   public boolean isPrefix() {
      return prefix;
   }

   public void setPrefix(boolean prefix) {
      this.prefix = prefix;
   }

   public boolean isText() {
      return text;
   }

   public void setText(boolean text) {
      this.text = text;
   }

   public int doStartTag() throws JspException {
      Object obj = pageContext.getAttribute(collection);
      if ( obj == null || !(obj instanceof CollectionClient) )
         throw new JspException("No Collection named '" + collection + "' has been declared");
      CollectionClient col = (CollectionClient)obj;

      try {
         if ( name != null ) {
            if ( text )
               pageContext.setAttribute(name, col.getDocumentAsText(key));
            else
               pageContext.setAttribute(name, col.getDocument(key));
         }
         else {
            String document = col.getDocumentAsText(key);
            if ( !prefix && document.startsWith("<?xml ") ) {
               int idx = document.indexOf('\n');
               if ( idx != -1 )
                  document = document.substring(idx + 1);
            }
            pageContext.getOut().print(document);
         }
      }
      catch ( Exception e ) {
         throw new JspException("Can't read Document '" + key + "' from Collection '" + collection + "'", e);
      }

      return SKIP_BODY;
   }
}
