package com.dbxml.db.enterprise.jsp;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: SetDocumentTag.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;

import com.dbxml.db.client.CollectionClient;

/**
 * SetDocumentTag updates the tag's BodyContent into the specified
 * CollectionClient using the specified key.
 * <br>
 * <br>
 * Example:
 * <br>
 * <pre>   &lt;dbxml:set collection="myCol" key="myDoc.xml">
 *      &lt;newdoc>
 *         Some new value
 *      &lt;/newdoc>
 *   &lt;/dbxml:set></pre>
 */

public class SetDocumentTag extends BodyTagSupport {
   static final String COLLECTION = "collection";
   static final String KEY = "key";

   private String collection;
   private String key;

   public SetDocumentTag() {
   }

   public String getCollection() {
      return collection;
   }

   public void setCollection(String collection) {
      this.collection = collection;
   }

   public String getKey() {
      return key;
   }

   public void setKey(String key) {
      this.key = key;
   }

   public int doEndTag() throws JspException {
      Object obj = pageContext.getAttribute(collection);
      if ( obj == null || !(obj instanceof CollectionClient) )
         throw new JspException("No Collection named '" + collection + "' has been declared");
      CollectionClient col = (CollectionClient)obj;

      try {
         BodyContent content = getBodyContent();
         String doc = content.getString().trim();
         col.setDocumentAsText(key, doc);
      }
      catch ( Exception e ) {
         throw new JspException("Can't set Document '" + key + "' on Collection '" + collection + "'", e);
      }

      return SKIP_BODY;
   }
}
