package com.dbxml.util;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ObjectQueue.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */

import java.io.Serializable;

/**
 * ObjectQueue is a simple linked list implemention that can be used
 * for FIFO queuing.  The benefit of ObjectQueue over Sun's Queue
 * implementation is that it is not an extension of ArrayList and doesn't
 * suffer from ArrayList's resizing performance limitations.
 */

public final class ObjectQueue implements Serializable {
   static final long serialVersionUID = 2573729619402197579L;

   private ObjectQueueItem first;
   private ObjectQueueItem last;
   private int size;

   public synchronized void add(Object value) {
      size++;
      ObjectQueueItem q = new ObjectQueueItem(value);
      if ( last != null )
         last.next = q;
      last = q;
      if ( first == null )
         first = q;
   }

   public synchronized Object remove() {
      ObjectQueueItem q = first;
      if ( q != null ) {
         size--;
         first = q.next;
         if ( first == null )
            last = null;
         return q.value;
      }
      else
         return null;
   }

   public boolean isEmpty() {
      return size == 0;
   }

   public int size() {
      return size;
   }

   public synchronized Object peek() {
		if ( first != null )
			return first.value;
		else
			return null;
   }

   public synchronized void clear() {
      size = 0;
      first = null;
      last = null;
   }


   /**
    * ObjectQueueItem
    */

   private class ObjectQueueItem implements Serializable {
      public Object value;
      public ObjectQueueItem next;

      public ObjectQueueItem(Object value) {
         this.value = value;
      }
   }
}

