package com.dbxml.util;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Poolable.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */

/**
 * Poolable is an interface that must be implemented by Objects that are
 * capable of being managed in a pooled fashion by dbXML.
 *
 * @see com.dbxml.util.ObjectPool
 */

public interface Poolable {
   /**
    * setPool passes a reference to the originating ObjectPool for this
    * Object.  This will allow the Poolable Object to call back to the
    * ObjectPool in order to be reclaimed.
    *
    * @param pool The parent ObjectPool
    */
   void setPool(ObjectPool pool);

   /**
    * reclaim prompts the Poolable object to return itself to the
    * ObjectPool.  A Poolable implementation should perform any
    * necessary cleanup before calling pool.putObject to return
    * itself to the ObjectPool.
    */
   void reclaim();
}
