package com.dbxml.util;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: dbXMLException.java,v 1.5 2004/02/12 00:17:58 bradford Exp $
 */

/**
 * A dbXMLException is the base class for all dbXML related
 * Exceptions.  This class allows an optional fault code, but no
 * specific fault codes are defined at this level.
 */

public class dbXMLException extends Exception {
   public int faultCode;

   public dbXMLException() {
      this(0, null, null);
   }

   public dbXMLException(int faultCode) {
      this(faultCode, null, null);
   }

   public dbXMLException(int faultCode, Throwable wrapped) {
      this(faultCode, null, wrapped);
   }

   public dbXMLException(int faultCode, String message) {
      this(faultCode, message, null);
   }

   public dbXMLException(Throwable wrapped) {
      this(0, null, wrapped);
   }

   public dbXMLException(String message) {
      this(0, message, null);
   }

   public dbXMLException(String message, Throwable wrapped) {
      this(0, message, wrapped);
   }

   public dbXMLException(int faultCode, String message, Throwable wrapped) {
      super(mergeMessages(message, wrapped), wrapped);
      this.faultCode = faultCode;
   }

   public int getFaultCode() {
      return faultCode;
   }

   private static String mergeMessages(String message, Throwable wrapped) {
      if ( wrapped != null ) {
         StringBuffer sb = new StringBuffer();

         if ( message != null )
            sb.append(message);

         String wrappedMsg = null;
         while ( wrappedMsg == null && wrapped != null ) {
            wrappedMsg = wrapped.getMessage();
            if ( wrappedMsg == null )
               wrapped = wrapped.getCause();
         }

         if ( wrappedMsg != null ) {
            if ( sb.equals(wrappedMsg) )
               return wrappedMsg;

            if ( sb.length() > 0 )
               sb.append('\n');

            sb.append(wrappedMsg);
         }

         return sb.toString();
      }
      else
         return message;
   }
}

