package com.dbxml.xml.dtsm;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: DocumentTable.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */

import com.dbxml.db.core.data.Value;
import com.dbxml.xml.SymbolTable;

/**
 * DocumentTable
 */

public interface DocumentTable {
   /**
    * getSymbolTable returns the SymbolTable associated with
    * this DocumentTable.
    *
    * @return The SymbolTable
    */
   SymbolTable getSymbolTable();

   /**
    * getEntryCount returns the number of entries in this DocumentTable.
    *
    * @return The DocumentEntry count
    */
   int getEntryCount();

   /**
    * getEntry returns the DocumentTableEntry associated
    * with the specified index.
    *
    * @param idx The DocumentTableEntry to retrieve
    * @return The DocumentTableEntry
    */
   DocumentTableEntry getEntry(int idx);

   /**
    * getValueCount returns the number of values in this
    * DocumentTable.
    *
    * @return The Value count
    */
   int getValueCount();

   /**
    * getValueEntry returns the ValueTableEntry associated
    * with the specified index.
    *
    * @param idx The ValueTableEntry to retrieve
    * @return The ValueTableEntry
    */
   ValueTableEntry getValueEntry(int idx);

   /**
    * getValue returns the Value associated with the
    * specified index.
    *
    * @param idx The Value to retrieve
    * @return The Value
    */
   Value getValue(int idx);
}
